﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.Sql; 
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using DotNetOpenAuth.OpenId;
using DotNetOpenAuth.OpenId.Extensions.AttributeExchange;
using DotNetOpenAuth.OpenId.Extensions.SimpleRegistration; 
using DotNetOpenAuth.OpenId.RelyingParty;
using OpenIdRelyingPartyWebForms; 

public partial class SSOLogin : System.Web.UI.Page
{
    #region ตรงนี้เป็น code ที่ไว้ใช้จัดการกับเรื่องการทำ OpenID นะครับ

    private const string RolesAttribute = "http://samples.dotnetopenauth.net/sso/roles";

    // บรรทัดนี้ต้องแก้ ให้เป็น OpenID Provider ที่ต้องการใช้งาน
    private const string OpenIdProviderURL = "http://testopenid.ega.or.th/";


    private static OpenIdRelyingParty relyingParty = new OpenIdRelyingParty();

    static SSOLogin()
    {
		// Configure the RP to only allow assertions from our trusted OP endpoint.
        EndpointSelector ep = new EndpointSelector(EndpointFilter);
        relyingParty.EndpointFilter = ep; 
	}

    protected static bool EndpointFilter(IProviderEndpoint ipep) 
    {
        return ipep.Uri.AbsoluteUri == OpenIdProviderURL + "server.aspx";
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        //ทำการสร้าง url ของหน้าที่ต้องการให้ส่งกลับ โดยในที่นี้ให้ส่งกลับมายังหน้าเดิม
        string[] segments = Request.Url.Segments;
        string pathToPage = ""; 
        for(int i=0; i<segments.Length-1; i++) {
            pathToPage += segments[i]; 
        } 
        UriBuilder returnToBuilder = new UriBuilder(Request.Url);
        returnToBuilder.Path = pathToPage + "SSOLogin.aspx";
        returnToBuilder.Query = "";
        returnToBuilder.Fragment = null;
        Uri returnTo = returnToBuilder.Uri;
        returnToBuilder.Path = "/";

        //บันทึก Url ลงใน Realm
        Realm realm = returnToBuilder.Uri;

        //ถ้าไม่เคย Request มาก่อน respones จะเป็น null
        IAuthenticationResponse response = relyingParty.GetResponse();
        if (response == null)
        {
            //เพื่อทำ Single Sign-On เราจะส่งผู้ใช้ไปยัง OpenID Provider ของ ICT โดยไม่ให้ผู้ใช้เลือก
            try
            {
                IAuthenticationRequest request = relyingParty.CreateRequest(
                    OpenIdProviderURL,
                    realm,
                    returnTo);

                // Attributes ที่ต้องการร้องขอจาก OpenID Provider ของ ICT
                // โดยใช้หลักการ AttributeExcahnge (AX)B
                var ax = new FetchRequest();
                ax.Attributes.Add(new AttributeRequest(WellKnownAttributes.Contact.Email, true));
                ax.Attributes.Add(new AttributeRequest(WellKnownAttributes.Name.FullName, true));
                ax.Attributes.Add(new AttributeRequest(WellKnownAttributes.Name.Alias, true));
                //ax.Attributes.Add(new AttributeRequest("http://www.egov.go.th/2012/identifier/citizenid", true));
                ax.Attributes.Add(new AttributeRequest("http://www.egov.go.th/2012/identifier/usertype", true));
                //ax.Attributes.Add(new AttributeRequest(WellKnownAttributes.Contact.Phone.Mobile, true));
                request.AddExtension(ax);

                //ส่ง Request ไปยัง OpenID Provider ของ ICT       
                request.RedirectToProvider();
            }
            catch(Exception eNull)
            {
                errorLabel.Text = @"ไม่พบ OpenID end point สาเหตุอาจมาจากการที่ท่านระบุ url ผิด หรือ ท่านไม่ได้ทำการเพิ่ม 
                                        SSL Certificate ไปยัง Trusted Root Store บนเครื่อง Server <br/><br/>" +
                                  eNull.Message + "<br/><br/>" + eNull.StackTrace +
                                  "<br/><br/>" + eNull.InnerException + "<br/><br/>" + eNull.Data; 
            }
        }
        else
        {
            switch (response.Status)
            {
                case AuthenticationStatus.Canceled:
                    // ผู้ใช้ยกเลิก
                    break;
                case AuthenticationStatus.Failed:
                    // ผู้ใช้ Authenticate ไม่ผ่าน
                    break;
                case AuthenticationStatus.Authenticated:

                    // ทำการดึงข้อมูลที่ได้รับมาจาก OpenID Provider ออกมา
                    FetchResponse fetchResponse = response.GetExtension<FetchResponse>();
                    State.FetchResponse = fetchResponse;
                    string claimed = response.FriendlyIdentifierForDisplay;
                    string userName = State.FetchResponse.Attributes[WellKnownAttributes.Name.Alias].Values[0];
                    string fullName = State.FetchResponse.Attributes[WellKnownAttributes.Name.FullName].Values[0];
                    //string identifier = State.FetchResponse.Attributes["http://www.egov.go.th/2012/identifier/citizenid"].Values[0];
                    string email = State.FetchResponse.Attributes[WellKnownAttributes.Contact.Email].Values[0];
                    string userType = State.FetchResponse.Attributes["http://www.egov.go.th/2012/identifier/usertype"].Values[0];
                    //string identifier =  State.FetchResponse.Attributes[WellKnownAttributes.Contact.Phone.Mobile].Values[0];
                    string identifier = "";
                    UserMappingAndAuthorization(claimed, userName, fullName, email, identifier, userType); 

                    break;
                default:
                    break;
            }
        }
    }
    #endregion 

    #region code ในส่วนนี้ตรงนี้ขึ้นอยู่แต่ละกับ e-Service
    /// <summary>
    /// ประเภทผู้ใช้(userType) ถูกแบ่งออกเป็น 4 ประเภทดังนี้
    /// - กรณีที่ประเภทผู้ใช้งาน = 1: ผู้ใช้งานเป็นบุคคลทั่วไป รหัสประจำตัวผู้ใช้งานจะเป็น เลขประจำตัวประชาชน 13 หลัก
    /// - กรณีที่ประเภทผู้ใช้งาน = 2: ผู้ใช้งานเป็นนิติบุคคล รหัสประจำตัวผู้ใช้งานจะเป็น เลขประจำตัวนิติบุคคล 
    /// - กรณีที่ประเภทผู้ใช้งาน = 3: ผู้ใช้งานเป็นชาวต่างชาติ รหัสประจำตัวผู้ใช้งานจะเป็น รหัสประเทศที่ออก ตามด้วย - และตามด้วยหมายเลข passport เช่น USA-C00001549 
    /// - กรณีที่ประเภทผู้ใช้งาน = 4: ผู้ใช้งานเป็นเจ้าหน้าที่รัฐ รหัสประจำตัวผู้ใช้งานจะเป็น เลขประจำตัวประชาชน 13 หลัก
    /// 
    /// ผู้พัฒนาต้อง implement Method นี้ เพื่อตรวจสอบว่า ผู้ใช้งานที่มีรหัสดังกล่าวเป็นสมาชิกของระบบ e-Service ของท่านอยู่แล้วหรือไม่ 
    /// 1) ถ้าเป็นสมาชิก Method นี้ ก็ควรจะ load ข้อมูลต่างๆ ที่จำเป็นสำหรับผู้ใช้ท่านนั้นๆ ใส่ไว้ใน session หรือ อื่นๆ ตามที่ท่านใช้ในหน้าจอ login ปกติ 
    ///    จากนั้นจึง redirect ผู้ใช้งาน ไปยังหน้าจอหลักของบริการ
    /// 2) ถ้าไม่เป็นสมาชิก ท่านอาจจะให้ผู้ใช้งานท่านนี้ทำการสมัครสมาชิกเสียก่อน โดย redirect ไปยังหน้า SSORegister (ซึ่งรายละเอียดเพิ่มเติมจะได้กล่าวถึงในหน้า code ดังกล่าว ต่อไป)
    /// 
    /// </summary>
    /// <param name="userIdentifier"></param>
    
    protected void UserMappingAndAuthorization(string openId, string userName, string fullName, string email, string userIdentifier, string userType)
    {
        errorLabel.Text = String.Format("OpenId : {0} <br/>", openId) + String.Format("Username : {0} <br/>", userName);
        errorLabel.Text += String.Format("Name : {0} <br/>", fullName) + String.Format("E-mail : {0} <br/>", email);
        errorLabel.Text += String.Format("Identifier : {0} <br/>", userIdentifier) + String.Format("User type : {0} <br/>", userType);
    }

    #endregion 

  
}
