﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml.XPath;
using System.Reflection;
using DotNetOpenAuth.OpenId;
using DotNetOpenAuth.OpenId.Extensions.AttributeExchange;
using DotNetOpenAuth.OpenId.RelyingParty;
using DotNetOpenAuth.OpenId.Extensions.SimpleRegistration;
using OpenIdRelyingPartyWebForms;
using DotNetOpenAuth.OAuth;
using DotNetOpenAuth.Messaging;
using DotNetOpenAuth;
using DotNetOpenAuth.ApplicationBlock;
using DotNetOpenAuth.OAuth.ChannelElements;
using DotNetOpenAuth.OAuth.Messages;
// lib ในการดึงค่าออกจาก xml ของสรอ.
using EGA.EGA_CAS.Util;
using EGA.EGA_CAS.Util.Entity;
using EGA.EGA_CAS.SSO.Library;

public partial class SSORegister : System.Web.UI.Page
{
    // แก้ 2 ตัวนี้ตามชื่อ e-service ต่างๆ
    // ซึ่งทางสรอ.จะเป็นคนกำหนด 2 ตัวนี้ให้
    private const string consumerKey = "sampleconsumer";
    private const string consumerSecret =  SecretUtil.encodeSecret("samplesecret");

    #region ตรงนี้เป็น code ที่ไว้ใช้จัดการกับเรื่องการทำ OAuth นะครับ

    // Url ในการขอ xml
    private const string xmlUrl = "http://testopenid.ega.or.th/XmlUserInfo.aspx";
    // Url ของ OAuth Provider
    private const string oAuthUrl = "http://testopenid.ega.or.th/OAuth.ashx";
   
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            //OAuth
            if (Session["WcfTokenManager"] != null)
            {
                //เริ่มสร้าง OAuth Request 
                WebConsumer consumer = this.CreateConsumer();
                AuthorizedTokenResponse accessTokenMessage = consumer.ProcessUserAuthorization();
                if (accessTokenMessage != null)
                {
                    Session["WcfAccessToken"] = accessTokenMessage.AccessToken;
                    Session.Remove("WcfTokenManager");

                    //นำ Access Token ไปแลก xml แล้ว bind เข้าสู่ Registeration form
                    string path = String.Format("{0}?AccessToken={1}", xmlUrl, Server.UrlEncode(accessTokenMessage.AccessToken));
                   
                    Response.Redirect(path);

                }
                else
                {
                   //เกิด Error ไม่ได้รับ Access Token
                }
            }
            else
            {
                WebConsumer consumer = this.CreateConsumer();
                UriBuilder callback = new UriBuilder(Request.Url);
                callback.Query = null;

                Dictionary<string, string> requestParams = new Dictionary<string, string>();
                requestParams.Add("scope", "");

                UserAuthorizationRequest response = consumer.PrepareRequestUserAuthorization(callback.Uri, requestParams, null);
                consumer.Channel.Send(response);
            }
        }
    }

    private int ConvertToInt(string input)
    {
        int result = 0;
        try
        {
            result = Convert.ToInt32(input);
        }
        catch
        {
            result = 1;
        }
        return result;
    }

    private WebConsumer CreateConsumer()
    {
        DotNetOpenAuth.ApplicationBlock.InMemoryTokenManager tokenManager = Session["WcfTokenManager"] as DotNetOpenAuth.ApplicationBlock.InMemoryTokenManager;
        if (tokenManager == null)
        {
            tokenManager = new DotNetOpenAuth.ApplicationBlock.InMemoryTokenManager(consumerKey, consumerSecret);
            Session["WcfTokenManager"] = tokenManager;
        }
        MessageReceivingEndpoint oauthEndpoint = new MessageReceivingEndpoint(
            new Uri(oAuthUrl),
            HttpDeliveryMethods.PostRequest);

        ServiceProviderDescription spd = new ServiceProviderDescription();
        spd.RequestTokenEndpoint = oauthEndpoint;
        spd.UserAuthorizationEndpoint = oauthEndpoint;
        spd.AccessTokenEndpoint = oauthEndpoint;
        spd.ProtocolVersion = DotNetOpenAuth.OAuth.ProtocolVersion.V10a;
        spd.TamperProtectionElements = new DotNetOpenAuth.Messaging.ITamperProtectionChannelBindingElement[] {
					new HmacSha1SigningBindingElement(),
				};
        WebConsumer consumer = new WebConsumer(spd, tokenManager);

        return consumer;
    }

    #endregion

    #region code ในส่วนนี้ตรงนี้ขึ้นอยู่แต่ละกับ e-Service

    /// <summary>
    /// Method bindValueToPage จะรับตัวแปร 1 ตัว คือ SSOUserInfo โดย method นี้จะถูกเรียกใช้โดย Page_Load event
    /// 
    /// ผู้พัฒนาต้องนำค่าจาก  SSOUserInfo ไปใส่ใน Register Form ที่ผู้พัฒนาทำขึ้น เพื่อให้ผู้ใช้ไม่ต้องกรอกข้อมูล
    /// ที่มีอยู่แล้วในระบบ e-portal ลงใน Register Form อีก โดยผู้พัฒนาสามารถเข้าถึงข้อมูลผ่าน Properties
    /// ใน class SSOUserInfo ที่จะอธิบายต่อไปนี้
    /// 
    ///        Properities                       Data Type                   คำอธิบาย
    /// SSOUserInfo.UserName                      ProfileField               ชื่อผู้ใช้ในระบบ e-portal
    /// SSOUserInfo.Title                         VerifiedField              คำนำหน้า
    /// SSOUserInfo.FirstName                     VerifiedField              ชื่อภาษาไทย
    /// SSOUserInfo.LastName                      VerifiedField              นามสกุลภาษาไทย
    /// SSOUserInfo.DateOfBirth                   VerifiedField              วันเกิด
    /// SSOUserInfo.Gender                        VerifiedField              เพศ
    /// SSOUserInfo.MemberType                    ProfileField               ประเภทผู้ใช้ 
    ///                                                                      - กรณีที่ประเภทผู้ใช้งาน = Citizen: ผู้ใช้งานเป็นบุคคลทั่วไป รหัสประจำตัวผู้ใช้งานจะเป็น เลขประจำตัวประชาชน 13 หลัก
    ///                                                                      - กรณีที่ประเภทผู้ใช้งาน = JuristicPerson: ผู้ใช้งานเป็นนิติบุคคล รหัสประจำตัวผู้ใช้งานจะเป็น เลขประจำตัวนิติบุคคล 
    ///                                                                      - กรณีที่ประเภทผู้ใช้งาน = Foreigner : ผู้ใช้งานเป็นชาวต่างชาติ รหัสประจำตัวผู้ใช้งานจะเป็น รหัสประเทศที่ออก ตามด้วย - และตามด้วยหมายเลข passport เช่น USA-C00001549 
    ///                                                                      - กรณีที่ประเภทผู้ใช้งาน = GovernmentAgent : ผู้ใช้งานเป็นเจ้าหน้าที่รัฐ รหัสประจำตัวผู้ใช้งานจะเป็น เลขประจำตัวประชาชน 13 หลัก
    /// SSOUserInfo.IdentificationCode            VerifiedField              เลขแสดงตัวตน โดยแปรผันตาม ประเภทผู้ใช้ 
    /// SSOUserInfo.IssuedBy                      VerifiedField              สถานที่ออก
    /// SSOUserInfo.IssuedDate                    VerifiedField              วันที่ออก
    /// SSOUserInfo.ExpiryDate                    VerifiedField              วันหมดอายุ
    /// SSOUserInfo.Nationality                   VerifiedField              สัญชาติ
    /// SSOUserInfo.Occupation                    VerifiedField              อาชีพ
    /// SSOUserInfo.HouseNumber                   VerifiedField              บ้านเลขที่
    /// SSOUserInfo.VillageName                   VerifiedField              ชื่ออาคาร/หมู่บ้าน
    /// SSOUserInfo.Soi                           VerifiedField              ซอย
    /// SSOUserInfo.Road                          VerifiedField              ถนน
    /// SSOUserInfo.SubDistrict                   VerifiedField              แขวง/ตำบล
    /// SSOUserInfo.District                      VerifiedField              เขต/อำเภอ
    /// SSOUserInfo.Province                      VerifiedField              จังหวัด
    /// SSOUserInfo.PostCode                      VerifiedField              รหัสไปรษณีย์
    /// SSOUserInfo.GeoCode                       VerifiedField              รหัสพื้นที่ตามกระทรวงมหาดไทย
    /// SSOUserInfo.Phone                         VerifiedField              เบอร์โทรศัพท์
    /// SSOUserInfo.MobliePhone                   VerifiedField              เบอร์โทรศัพท์เคลื่อนที่
    /// SSOUserInfo.Email                         VerifiedField              E-mail Address
    /// SSOUserInfo.OrgName                       OrganizationField          หน่วยงาน
    /// SSOUserInfo.Ministry                      OrganizationField          กระทรวง
    /// SSOUserInfo.Department                    OrganizationField          กรม
    /// SSOUserInfo.Division                      OrganizationField          กอง
    /// 
    /// ProfileField.StringValue -> คืนค่าเป็น String ของ Properities
    /// VerifiedField.StringValue -> คืนค่าเป็น String ของ Properities
    /// OrganizationField.StringValue -> คืนค่าเป็น String ของ Properities
    /// 
    /// ProfileField.Value -> คืนค่าเป็น Object ของ Properities
    /// VerifiedField.Value -> คืนค่าเป็น Object ของ Properities
    /// OrganizationField.Value -> คืนค่าเป็น Object ของ Properities
    /// 
    /// VerifiedField.VerifiedLevel -> คืนค่า VerifiedLevel ของ Properities
    /// OrganizationField.VerifiedLevel -> คืนค่า VerifiedLevelของ Properities
    /// 
    /// OrganizationField.Code -> Code ของ กระทรวง/กรม/กอง/หน่วยงาน
    /// </summary>
    /// <param name="userIdentifier"></param>
    private void bindValueToPage(SSOUserInfo ssoUI)
    {
        //ตัวอย่าง
        txt_UserName.Text = ssoUI.IdentificationCode.StringValue;
        txt_FirstName.Text = ssoUI.FirstName.StringValue;
        txt_LastName.Text = ssoUI.LastName.StringValue;
        txt_Citizen.Text = ssoUI.IdentificationCode.StringValue;
        txt_Email.Text = ssoUI.Email.StringValue;
        txt_HouseNumber.Text = ssoUI.HouseNumber.StringValue;
        txt_Village.Text = ssoUI.VillageName.StringValue;
        txt_Soi.Text = ssoUI.Soi.StringValue;
        txt_Road.Text = ssoUI.Road.StringValue;
        txt_PostCode.Text = ssoUI.PostCode.StringValue;
        txt_Phone.Text = ssoUI.Phone.StringValue;
        //txt_Fax.Text = ssoUI.Fax;
        txt_MPhone.Text = ssoUI.MobliePhone.StringValue;

        Response.Write(ssoUI.FirstName.VerifiedLevel);
        Response.Write(ssoUI.DateOfBirth.StringValue);
        Response.Write(ssoUI.MemberType.StringValue);

    }

    //ลงทะเบียน
    protected void btn_Submit_Click(object sender, EventArgs e)
    { 
    
    }

    #endregion



    protected void Button1_Click(object sender, EventArgs e)
    {
      
    }
}
